from __future__ import annotations

from typing_extensions import Self

from .AFXDialog import AFXDialog
from .FXIcon import FXIcon
from .FXObject import FXObject

#: GUI has no components in standard locations.
GUI_IN_NONE: int = hash("GUI_IN_NONE")

#: GUI has components in the menubar.
GUI_IN_MENUBAR: int = hash("GUI_IN_MENUBAR")

#: GUI has components in the Tools pull down pane.
GUI_IN_TOOL_PANE: int = hash("GUI_IN_TOOL_PANE")

#: GUI has components in the toolbar.
GUI_IN_TOOLBAR: int = hash("GUI_IN_TOOLBAR")

#: GUI has components in the toolbox.
GUI_IN_TOOLBOX: int = hash("GUI_IN_TOOLBOX")


class AFXGuiObjectManager(FXObject):
    """This is a base class for management of GUI components found in the menubar, toolbar, and toolbox."""

    #: Used to destroy dialogs.
    ID_DESTROY_DIALOGS: int = hash("ID_DESTROY_DIALOGS")

    def __init__(self, source: Self):
        """Undefined copy constructor (this class does not have copy semantics).

        Parameters
        ----------
        source : Self
            Object to be copied.
        """

    def getDialog(self, widgetAlias: str):
        """Returns the dialog box with the specified widget key.

        Parameters
        ----------
        widgetAlias : str
            Dialog box alias.
        """

    def getKernelInitializationCommand(self):
        """Returns the command that should initialize the corresponding module or toolset in the kernel.

        Called by the module manager the first time the GUI module is switched into.
        """

    def hide(self, location: int):
        """Hides the GUI components in the menubar, toolbar, and toolbox.

        Parameters
        ----------
        location : int
            Location where GUI components are placed.
        """

    def registerAndShowDialog(self, dialog: AFXDialog):
        """Registers the given dialog box and its widget key with the manager and posts the dialog box.

        Parameters
        ----------
        dialog : AFXDialog
            Dialog box.
        """

    def registerAndShowModalDialog(self, dialog: AFXDialog):
        """Registers the given dialog box and its widget key with the manager and posts the dialog box as a
        modal dialog box.

        Parameters
        ----------
        dialog : AFXDialog
            Dialog box.
        """

    def registerShortcutFunction(
        self,
        text: str,
        tgt: FXObject,
        sel: int,
        ic: FXIcon | None = None,
        tipText: str = "''",
        displayedName: str = "''",
        typesToDisplay: int = 0,
    ):
        """Registers a shortcut function; this function will be available in the GUI so that users can assign it
        shortcut keys.

        Parameters
        ----------
        text : str
            Label used to identify the function in the GUI; To specify a shortcut, separate it from the label using "t".
        tgt : FXObject
            Message target.
        sel : int
            Messaged ID.
        ic : FXIcon | None
            Icon for the shortcut
        tipText : str
            Text used for buttom tooltip
        displayedName : str
            Name of the module to which this function belongs.
        typesToDisplay : int
            Flags specifying the types of objects displayed in the module; see AFXModuleGui.
        """

    def sendCommandString(self, command: str, writeToReplay: bool = True, writeToJournal: bool = False):
        """Sends the given command string (which can contain multiple commands, separated by command
        delimiters).

        Parameters
        ----------
        command : str
            Command string.
        writeToReplay : bool
            True if commands should be written to the replay file; False if not.
        writeToJournal : bool
            True if commands should be written to the journal file; False if not.
        """

    def show(self, location: int):
        """Shows the GUI components in the menubar, toolbar, and toolbox.

        Parameters
        ----------
        location : int
            Location where GUI components are placed.
        """

    def unregisterDialog(self, widgetAlias: str):
        """Unregisters the dialog box of the given widget key from the manager.

        Parameters
        ----------
        widgetAlias : str
            Dialog box alias.
        """



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class Monitor:
    """The Monitor object defines a degree of freedom to monitor.

    .. note::
        This object can be accessed by::

            import step
            mdb.models[name].steps[name].monitor

        The corresponding analysis keywords are:

        - MONITOR
    """

    #: A String specifying the name of the region to be monitored.
    node: str

    #: A SymbolicConstant specifying the degree of freedom to be monitored at the node.
    #: Possible values are:
    #:
    #: - U1
    #: - U2
    #: - U3
    #: - UR1
    #: - UR2
    #: - UR3
    #: - WARP
    #: - FLUID_PRESSURE
    #: - ELECTRICAL_POTENTIAL
    #: - NT11
    #: - NT30
    #: - NN11
    #: - NN30
    #:
    #: The NT identifiers are not available for mass diffusion. The NN identifiers are
    #: available only for mass diffusion.
    dof: SymbolicConstant

    #: An Int specifying the output frequency in increments. This argument is valid only for an
    #: Abaqus/Standard analysis.
    frequency: int

    @abaqus_method_doc
    def __init__(
        self,
        node: str,
        dof: Literal[
            C.NN11,
            C.NT,
            C.U3,
            C.NT30,
            C.UR2,
            C.ELECTRICAL_POTENTIAL,
            C.NT11,
            C.U1,
            C.UR3,
            C.WARP,
            C.FLUID_PRESSURE,
            C.UR1,
            C.NN30,
            C.U2,
        ],
        frequency: int,
    ):
        """This method creates a request for a degree of freedom to be monitored in a general or modal
        procedure.

        .. note::
            This function can be accessed by::

                mdb.models[name].steps[name].Monitor

        Parameters
        ----------
        node
            A String specifying the name of the region to be monitored.
        dof
            A SymbolicConstant specifying the degree of freedom to be monitored at the node.
            Possible values are:

            - U1
            - U2
            - U3
            - UR1
            - UR2
            - UR3
            - WARP
            - FLUID_PRESSURE
            - ELECTRICAL_POTENTIAL
            - NT11
            - NT30
            - NN11
            - NN30

            The NT identifiers are not available for mass diffusion. The NN identifiers are
            available only for mass diffusion.
        frequency
            An Int specifying the output frequency in increments. This argument is valid only for an
            Abaqus/Standard analysis.

        Returns
        -------
        Monitor
            A Monitor object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the Monitor object.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ....UtilityAndView.abaqusConstants import OFF, Boolean
from .MohrCoulombHardening import MohrCoulombHardening
from .TensionCutOff import TensionCutOff


@abaqus_class_doc
class MohrCoulombPlasticity:
    r"""The MohrCoulombPlasticity object specifies the extended Mohr-Coulomb plasticity model.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].mohrCoulombPlasticity
            import odbMaterial
            session.odbs[name].materials[name].mohrCoulombPlasticity

        The table data for this object are:

        - Friction angle (given in degrees), :math:`\phi`, at high confining pressure
          in the :math:`p-R_{m c} q` plane.
        - Dilation angle, :math:`\psi`, at high confining pressure in the :math:`p-R_{m w} q` plane.
        - Temperature, if the data depend on temperature.
        - Value of the first field variable, if the data depend on field variables.
        - Value of the second field variable.
        - Etc.

        The corresponding analysis keywords are:

        - MOHR COULOMB
    """

    #: A MohrCoulombHardening object.
    mohrCoulombHardening: MohrCoulombHardening = MohrCoulombHardening(((),))

    #: A TensionCutOff object.
    tensionCutOff: TensionCutOff = TensionCutOff(((),))

    @abaqus_method_doc
    def __init__(
        self,
        table: tuple,
        deviatoricEccentricity: float | None = None,
        meridionalEccentricity: float = 0,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
        useTensionCutoff: Boolean = OFF,
    ):
        r"""This method creates a MohrCoulombPlasticity object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].MohrCoulombPlasticity
                session.odbs[name].materials[name].MohrCoulombPlasticity

        Parameters
        ----------
        table
            A sequence of sequences of Floats specifying the items described below.
        deviatoricEccentricity
            None or a Float specifying the flow potential eccentricity in the deviatoric plane,
            :math:`e`; :math:`1 / 2 \leq e \leq 1.0`. If **deviatoricEccentricity** = None, Abaqus calculates the value using the
            specified Mohr-Coulomb angle of friction. The default value is None.
        meridionalEccentricity
            A Float specifying the flow potential eccentricity in the meridional plane, :math:`\epsilon`. The
            default value is 0.1.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        useTensionCutoff
            A Boolean specifying whether tension cutoff specification is needed. The default value
            is OFF.

        Returns
        -------
        MohrCoulombPlasticity
            A MohrCoulombPlasticity object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the MohrCoulombPlasticity object.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from typing_extensions import Self

from .FXWindow import FXWindow


class FXComposite(FXWindow):
    """Base composite."""

    def __init__(self, p: Self, opts: int = 0, x: int = 0, y: int = 0, w: int = 0, h: int = 0):
        """Constructor.

        Parameters
        ----------
        p : Self

        opts : int

        x : int

        y : int

        w : int

        h : int
        """

    def create(self):
        """Create server-side resources.

        Reimplemented from FXWindow. Reimplemented in FXColorSelector, FXComboBox, FXDirBox, FXDirList,
        FXDriveBox, FXFileList, FXFontSelector, FXGroupBox, FXIconList, FXImageView, FXList, FXListBox,
        FXMDIChild, FXPrintDialog, FXRootWindow, FXScrollWindow, FXShell, FXSpinner, FXTabBar, FXTable,
        FXText, FXToolbarShell, FXTooltip, FXTopWindow, FXTreeList, FXTreeListBox, AFXManagerMenuPane,
        AFXMainWindow, AFXPromptArea, AFXBaseTable, AFXColorButton, AFXColorFlyout, AFXComboBox, AFXDialog,
        AFXFloatSpinner, AFXListBox, AFXNote, AFXOptionTreeItem, AFXPrimFloatSpinner, AFXSpinner, AFXTable,
        AFXTextField, and AFXVerticalAligner.
        """

    def destroy(self):
        """Destroy server-side resources.

        Reimplemented from FXWindow. Reimplemented in FXComboBox, FXDirBox, FXDirList, FXDriveBox,
        FXFileList, FXListBox, FXRootWindow, FXTreeList, FXTreeListBox, AFXColorFlyout, and AFXTable.
        """

    def detach(self):
        """Detach server-side resources.

        Reimplemented from FXWindow. Reimplemented in FXComboBox, FXDirBox, FXDirList, FXDriveBox,
        FXFileList, FXGroupBox, FXIconList, FXImageView, FXList, FXListBox, FXMDIChild, FXRootWindow,
        FXTable, FXText, FXTooltip, FXTopWindow, FXTreeList, FXTreeListBox, AFXBaseTable, AFXColorFlyout,
        AFXNote, and AFXTable.
        """

    def getDefaultHeight(self):
        """Return default height.

        Reimplemented from FXWindow. Reimplemented in FX4Splitter, FXComboBox, FXDockSite, FXGroupBox,
        FXHorizontalFrame, FXList, FXListBox, FXMDIChild, FXMatrix, FXPacker, FXPopup, FXRootWindow,
        FXScrollArea, FXSpinner, FXSplitter, FXStatusbar, FXSwitcher, FXTabBar, FXTabBook, FXTable, FXText,
        FXToolbar, FXToolbarShell, FXTooltip, FXTopWindow, FXTreeList, FXTreeListBox, FXVerticalFrame,
        AFXMainWindow, AFXToolbarGroup, AFXBaseTable, AFXList, AFXOptionTreeList, AFXPrimFloatSpinner,
        AFXSlider, AFXTable, AFXTreeTable, and AFXVerticalAligner.
        """

    def getDefaultWidth(self):
        """Return default width.

        Reimplemented from FXWindow. Reimplemented in FX4Splitter, FXComboBox, FXDockSite, FXGroupBox,
        FXHorizontalFrame, FXList, FXListBox, FXMDIChild, FXMatrix, FXPacker, FXPopup, FXRootWindow,
        FXScrollArea, FXSpinner, FXSplitter, FXStatusbar, FXSwitcher, FXTabBar, FXTabBook, FXTable, FXText,
        FXToolbar, FXToolbarShell, FXTooltip, FXTopWindow, FXTreeList, FXTreeListBox, FXVerticalFrame,
        AFXMainWindow, AFXToolbarGroup, AFXBaseTable, AFXOptionTreeItem, AFXOptionTreeList,
        AFXPrimFloatSpinner, AFXSlider, AFXTable, AFXTextField, AFXTreeTable, and AFXVerticalAligner.
        """

    def maxChildHeight(self):
        """Return the height of the tallest child window."""

    def maxChildWidth(self):
        """Return the width of the widest child window.

        By clicking on Send, you accept that Dassault Systèmes will process your personal data and may
        contact you for further information. [Privacy Policy](
        https://www.3ds.com/privacy-policy).
        Total Results: Results per page This page cannot be found. The page might not exist or is temporarily unavailable. Try again or try searching for the topic.
        """



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class CrushStressVelocityFactor:
    """The CrushStressVelocityFactor object defines how the approach velocity at a crushing interface influences
    a material's resistance to crushing.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].crushStress.crushStressVelocityFactor
            import odbMaterial
            session.odbs[name].materials[name].crushStress.crushStressVelocityFactor

        The table data for this object are:

        - Scaling factor.
        - Relative velocity.

        The corresponding analysis keywords are:

        - CRUSH STRESS VELOCITY FACTOR

    .. versionadded:: 2022
        The ``CrushStressVelocityFactor`` class was added.
    """

    #: A sequence of sequences of Floats specifying the items described below.
    crushStressVelocityFactorTable: tuple[tuple[float, ...], ...] = ()

    @abaqus_method_doc
    def __init__(self, crushStressVelocityFactorTable: tuple[tuple[float, ...], ...]):
        """This method creates a CrushStressVelocityFactor object.

        Parameters
        ----------
        crushStressVelocityFactorTable
            A sequence of sequences of Floats specifying the items described below.
        """
        self.crushStressVelocityFactorTable = crushStressVelocityFactorTable

    @abaqus_method_doc
    def setValues(self, crushStressVelocityFactorTable: tuple[tuple[float, ...], ...] = ()):
        """This method creates a CrushStressVelocityFactor object.

        Parameters
        ----------
        crushStressVelocityFactorTable
            A sequence of sequences of Floats specifying the items described below.
        """
        self.crushStressVelocityFactorTable = crushStressVelocityFactorTable



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc

from .RayleighDampingComponentArray import RayleighDampingComponentArray


@abaqus_class_doc
class RayleighDamping:
    """A RayleighDamping object contains Rayleigh Damping parameters.

    .. note::
        This object can be accessed by::

            import step
            mdb.models[name].steps[name].rayleighDamping
    """

    #: A RayleighDampingComponentArray object.
    components: RayleighDampingComponentArray = []



from __future__ import annotations

from .FXComposite import FXComposite
from .FXLabel import FXLabel
from .FXPopup import FXPopup

#: Automatically gray out when no target.
AFXFLYOUT_AUTOGRAY: int = hash("AFXFLYOUT_AUTOGRAY")

#: Automatically hide when no target.
AFXFLYOUT_AUTOHIDE: int = hash("AFXFLYOUT_AUTOHIDE")

#: Toolbar style button.
AFXFLYOUT_TOOLBAR: int = hash("AFXFLYOUT_TOOLBAR")

#: Popup horizontal.
AFXFLYOUT_HORIZONTAL: int = hash("AFXFLYOUT_HORIZONTAL")

#: Normal flyout button.
AFXFLYOUT_NORMAL: int = hash("AFXFLYOUT_NORMAL")

#: Popup vertical.
AFXFLYOUT_VERTICAL: int = hash("AFXFLYOUT_VERTICAL")

#: Current item is always active.
AFXFLYOUT_RADIO: int = hash("AFXFLYOUT_RADIO")


class AFXFlyoutButton(FXLabel):
    """This class contains a button that acts like a regular FXButton when pressed and released quickly but
    displays a popup menu when pressed and held for a short time duration."""

    #: ID for the popup timer.
    ID_AFXFLYOUT_TIMER: int = hash("ID_AFXFLYOUT_TIMER")

    #: ID used when hiding flyout item.
    ID_HIDE_ITEM: int = hash("ID_HIDE_ITEM")

    def __init__(
        self,
        p: FXComposite,
        pup: FXPopup | None = None,
        act: int = 0,
        opts: int = AFXFLYOUT_NORMAL,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
        pl: int = 0,
        pr: int = 0,
        pt: int = 0,
        pb: int = 0,
    ):
        """Constructor.

        Parameters
        ----------
        p : FXComposite
            Parent widget.
        pup : FXPopup | None
            Popup containing flyout items.
        act : int
            Current button index (0-based).
        opts : int
            Options and hints.
        x : int
            X coordinate of origin.
        y : int
            Y coordinate of origin.
        w : int
            Width of the widget.
        h : int
            Height of the widget.
        pl : int
            Left padding (margin).
        pr : int
            Right padding (margin).
        pt : int
            Top padding (margin).
        pb : int
            Bottom padding (margin).
        """

    def canFocus(self):
        """Returns True (because a flyout button can receive focus).

        Reimplemented from FXWindow.
        """

    def create(self):
        """Creates the flyout button.

        Reimplemented from FXLabel.
        """

    def detach(self):
        """Detaches server-side resources for the flyout button.

        Reimplemented from FXLabel.
        """

    def disable(self):
        """Disables the flyout button.

        Reimplemented from FXLabel.
        """

    def enable(self):
        """Enables the flyout button.

        Reimplemented from FXLabel.
        """

    def getButtonStyle(self):
        """Returns the flyout button style."""

    def getCurrentItem(self):
        """Returns the current item."""

    def getPane(self):
        """Returns the popup menu."""

    def getState(self):
        """Returns the flyout button state."""

    def setButtonStyle(self, style: int):
        """Sets the flyout button style.

        Parameters
        ----------
        style : int
            Button style (see Flags for flyout button options.)
        """

    def setCurrentItem(self, index: int, setCheck: bool = False):
        """Sets the current item and depresses the button if setCheck is True. The specified item index is
        0-based, and only valid items are counted (items such as separators are not counted).

        Parameters
        ----------
        index : int
            Index.
        setCheck : bool
            Value of check button.
        """

    def setPane(self, pup: FXPopup):
        """Sets the popup menu.

        Parameters
        ----------
        pup : FXPopup
            Popup menu.
        """

    def setState(self, state: int):
        """Sets the flyout button state.

        Parameters
        ----------
        state : int
            State (see FXButton's Button state bits).
        """



from __future__ import annotations

from .AFXProcedure import AFXProcedure
from .AFXStep import AFXStep


class AFXEditSketchStep(AFXStep):
    """This class is used to provide pick steps in GUI procedures."""

    def __init__(self, owner: AFXProcedure, sketchName: str, prompt: str = "'Edit a sketch'"):
        """Constructor.

        Parameters
        ----------
        owner : AFXProcedure
            Procedure creating the step.
        sketchName : str
            Name of sketch to edit, blank if create.
        prompt : str
            Step's prompt displayed in prompt area.
        """

    def onCancel(self):
        """Called when the step is cancelled.

        Reimplemented from AFXStep.
        """

    def onExecute(self):
        """Called to execute the steps returned by getFirstStep and getNextStep.

        Reimplemented from AFXStep.
        """

    def onResume(self):
        """Called when the step is resumed.

        Reimplemented from AFXStep.
        """

    def onSuspend(self):
        """Called when the step is suspended.

        Reimplemented from AFXStep.
        """



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ...UtilityAndView.abaqusConstants import OFF, Boolean


@abaqus_class_doc
class GapConductance:
    """The GapConductance object specifies conductive heat transfer between closely adjacent (or contacting)
    surfaces.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].gapConductance
            import odbMaterial
            session.odbs[name].materials[name].gapConductance

        The table data for this object are:

        - Gap Conductance or Cohesive Separation.
        - Gap Clearance, Gap Pressure (if optional parameter pressureDependency is used), or Closure, :math:`c`
          (for coupled temperature-displacement gasket elements).
        - Average Temperature if the data depend on temperature.
        - Mass Flow Rate per unit area if the data depend on the average mass flow rate.
        - Value of the first field variable if the data depend on field variables.
        - Value of the second field variable.
        - Etc.

        The corresponding analysis keywords are:

        - GAP CONDUCTANCE

    .. versionadded:: 2021
        The ``GapConductance`` class was added.
    """

    @abaqus_method_doc
    def __init__(
        self,
        pressureDependency: Boolean = OFF,
        dependencies: int = 0,
        table: tuple = (),
    ):
        """This method creates a GapConductance object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].GapConductance
                session.odbs[name].materials[name].GapConductance

        Parameters
        ----------
        pressureDependency
            A Boolean specifying whether the data depend on pressure. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.
        table
            A sequence of sequences of Floats specifying the items described below.

        Returns
        -------
            A GapConductance object.

        Raises
        ------
        """
        ...

    @abaqus_method_doc
    def setValues(self):
        """This method modifies the GapConductance object."""
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import (
    ABAQUS,
    ANALYSIS,
    DEFAULT,
    ON,
    Boolean,
    SymbolicConstant,
)
from .Job import Job


@abaqus_class_doc
class Coexecution:
    """The Coexecution object contains a set of jobs as associated parameters to define a co-simulation
    analysis.

    .. note::
        This object can be accessed by::

            import job
            mdb.coexecutions[name]

        The corresponding analysis keywords are:

        - HEADING
        - PREPRINT
    """

    #: A String specifying the name of the new job. The name must be a valid Abaqus/CAE object
    #: name.
    name: str = ""

    #: A SymbolicConstant specifying the type of analysis to execute for the co-execution.
    #: Possible values are ANALYSIS, SYNTAXCHECK, RECOVER, and RESTART. The default value is
    #: ANALYSIS.
    type: SymbolicConstant = ANALYSIS

    #: A SymbolicConstant specifying the analysis product type of the main model for the
    #: co-execution. The default value is ABAQUS.
    #:
    #: .. versionchanged:: 2022
    #:     The ``masterAnalysisProduct`` attribute was changed to ``mainAnalysisProduct``.
    mainAnalysisProduct: SymbolicConstant = ABAQUS

    #: An Int specifying the number of hours to wait before submitting the co-execution. This
    #: argument is ignored if **queue** is set. The default value is 0.This argument works in
    #: conjunction with **waitMinutes**. **waitHours** and **atTime** are mutually exclusive.
    waitHours: int = 0

    #: An Int specifying the number of minutes to wait before submitting the job. This argument
    #: is ignored if **queue** is set. The default value is 0.This argument works in conjunction
    #: with **waitHours**. **waitMinutes** and **atTime** are mutually exclusive.
    waitMinutes: int = 0

    #: A SymbolicConstant specifying the status of the co-execution. Possible values are
    #: SUBMITTED, RUNNING, ABORTED, TERMINATED, COMPLETED, CHECK_SUBMITTED, CHECK_RUNNING, and
    #: CHECK_COMPLETED.If the **message** member of all the jobs are empty, **status** is set to
    #: NONE.
    status: SymbolicConstant

    #: A String specifying the name of the queue to which to submit the co-execution. The
    #: default value is an empty string. Note: You can use the **queue** argument when creating a
    #: Coexecution object on a Windows workstation; however, remote queues are available only
    #: on Linux platforms.
    queue: str = ""

    #: A String specifying the time at which to submit the co-execution. If **queue** is empty,
    #: the string syntax must be valid for the Linux ``at`` command. If **queue** is set, the
    #: syntax must be valid according to the system administrator. The default value is an
    #: empty string. Note: You can use the **atTime** argument when creating a Coexecution object
    #: on a Windows workstation; however, the ``at`` command is available only on Linux
    #: platforms.
    atTime: str = ""

    #: A repository of Job objects specifying the jobs that comprise this co-execution.
    jobs: dict[str, Job] = {}

    #: A tuple of Strings specifying the names of the secondary models for the co-execution.
    #:
    #: .. versionchanged:: 2022
    #:     The ``slaveModels`` attribute was changed to ``secondaryModels``.
    secondaryModels: tuple[str, ...] = ()

    #: A tuple of SymbolicConstants specifying the analysis product types of the secondary
    #: models for the co-execution. The default value is an empty sequence.
    #:
    #: .. versionchanged:: 2022
    #:     The ``slaveAnalysisProducts`` attribute was changed to ``secondaryAnalysisProducts``.
    secondaryAnalysisProducts: SymbolicConstant

    #: A String specifying the name of the main model for the co-execution.
    #:
    #: .. versionchanged:: 2022
    #:     The ``masterModel`` attribute was changed to ``mainModel``.
    mainModel: str = ""

    #: A SymbolicConstant specifying the type of license type being used in case of DSLS
    #: SimUnit license model. Possible values are DEFAULT, TOKEN, and CREDIT. The default value
    #: is DEFAULT.If the license model is not DSLS SimUnit then the licenseType is not
    #: available.
    #:
    #: .. versionadded:: 2022
    #:     The ``licenseType`` attribute was added.
    licenseType: SymbolicConstant = DEFAULT

    @abaqus_method_doc
    def kill(self):
        """This method kills the analysis of a co-execution."""
        ...

    @abaqus_method_doc
    def submit(
        self,
        consistencyChecking: Boolean = ON,
        datacheckJob: Boolean = False,
        continueJob: Boolean = False,
    ):
        """This method submits a co-execution for analysis.

        Parameters
        ----------
        consistencyChecking
            A Boolean specifying whether to perform consistency checking for the individual jobs.
            The default value is ON. It is not recommended to turn the consistency checking off
            unless you are absolutely sure the models are all consistent.
        datacheckJob
            A Boolean specifying whether to run the co-execution as a datacheck analysis. The
            default value is False. The **datacheckJob** and **continueJob** arguments cannot both be
            True.
        continueJob
            A Boolean specifying whether to run the co-execution as a continuation analysis. The
            default value is False. The **datacheckJob** and **continueJob** arguments cannot both be
            True.
        """
        ...

    @abaqus_method_doc
    def writeInput(self, consistencyChecking: Boolean = ON):
        """This method writes an input file for each analysis in the co-execution.

        Parameters
        ----------
        consistencyChecking
            A Boolean specifying whether to perform consistency checking for the individual jobs.
            The default value is ON. It is not recommended to turn the consistency checking off
            unless you are absolutely sure the models are all consistent.
        """
        ...

    @abaqus_method_doc
    def waitForCompletion(self):
        """This method interrupts the execution of the script until the end of all the analyses.

        If you call the waitForCompletion method and the **status** member is neither SUBMITTED nor RUNNING,
        Abaqus assumes the analysis has either completed or aborted and returns immediately.
        """
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import ON, Boolean


@abaqus_class_doc
class EngineeringFeatureDisplayOptions:
    """The EngineeringFeatureDisplayOptions object stores settings that specify how assemblies are to be
    displayed in a particular viewport when session.viewports[name].assemblyDisplay.engineeringFeatures=ON The
    EngineeringFeatureDisplayOptions object has no constructor. When you create a new viewport, the settings are
    copied from the current viewport.

    .. note::
        This object can be accessed by::

            session.viewports[name].assemblyDisplay.engineeringFeatureOptions
            session.viewports[name].layers[name].assemblyDisplay.engineeringFeatureOptions
            import part
            session.viewports[name].layers[name].partDisplay.engineeringFeatureOptions
            session.viewports[name].partDisplay.engineeringFeatureOptions
    """

    @abaqus_method_doc
    def setValues(
        self,
        pointMassInertia: Boolean = ON,
        nonstructuralMass: Boolean = ON,
        heatCapacitance: Boolean = ON,
        contourIntegral: Boolean = ON,
        springToGround: Boolean = ON,
        twoPointSpring: Boolean = ON,
    ):
        """This method modifies the EngineeringFeatureDisplayOptions object.

        Parameters
        ----------
        pointMassInertia
            A Boolean specifying whether point mass inertia symbols are shown. The default value is
            ON.
        nonstructuralMass
            A Boolean specifying whether nonstructural mass symbols are shown. The default value is
            ON.
        heatCapacitance
            A Boolean specifying whether heat capacitance symbols are shown. The default value is
            ON.
        contourIntegral
            A Boolean specifying whether contour integral symbols are shown. The default value is
            ON.
        springToGround
            A Boolean specifying whether spring-to-ground symbols are shown. The default value is
            ON.
        twoPointSpring
            A Boolean specifying whether two-point spring symbols are shown. The default value is
            ON.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from typing_extensions import Self

from .FXObject import FXObject


class FXObjectList:
    """List of pointers to objects."""

    def __init__(self, orig: Self):
        """Copy constructor.

        Parameters
        ----------
        orig : Self
        """

    def append(self, p: FXObject):
        """Append element.

        Parameters
        ----------
        p : FXObject
        """

    def clear(self):
        """Remove all elements."""

    def findb(self, p: FXObject, pos: int = 2147483647):
        """Find object in list, searching backward; return position or -1.

        Parameters
        ----------
        p : FXObject

        pos : int
        """

    def findf(self, p: FXObject, pos: int = 0):
        """Find object in list, searching forward; return position or -1.

        Parameters
        ----------
        p : FXObject

        pos : int
        """

    def insert(self, pos: int, p: FXObject):
        """Insert element at certain position.

        Parameters
        ----------
        pos : int

        p : FXObject
        """

    def no(self):
        """Return number of elements."""

    def remove(self, pos: int):
        """Remove element at pos.

        Parameters
        ----------
        pos : int
        """

    def size(self):
        """Return size of list.

        By clicking on Send, you accept that Dassault Systèmes will process your personal data and may
        contact you for further information. [Privacy Policy](
        https://www.3ds.com/privacy-policy).
        Total Results: Results per page
        """



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import MILLING_REGION, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .GeometricRestriction import GeometricRestriction


@abaqus_class_doc
class TopologyMillingControl(GeometricRestriction):
    """The TopologyMillingControl object defines a topology milling control geometric restriction. The
    TopologyMillingControl object is derived from the GeometricRestriction object.

    .. note::
        This object can be accessed by::

            import optimization
            mdb.models[name].optimizationTasks[name].geometricRestrictions[name]

    .. versionadded:: 2022
        The ``TopologyMillingControl`` class was added.
    """

    #: A String specifying the geometric restriction repository key.
    name: str

    #: A tuple of VertexArray objects of length 2 specifying the milling directions. Each point
    #: can be specified through a tuple of coordinates instead of through a ConstrainedSketchVertex.
    millingDirections: tuple

    #: A Region object specifying the region to which the geometric restriction is applied.
    region: Region

    #: None or a DatumCsys object specifying the local coordinate system of the
    #: **millingDirections**. If **csys** = None, the global coordinate system is used. When this
    #: member is queried, it returns an Int indicating the identifier of the DatumCsys. The
    #: default value is None.
    csys: int | None = None

    #: The SymbolicConstant MILLING_REGION or a Region object specifying the milling check
    #: region. If the value is MILLING_REGION, the value of **region** is used as both the
    #: milling control region and the milling check region. The default value is
    #: MILLING_REGION.
    millingCheckRegion: SymbolicConstant = MILLING_REGION

    #: A Float specifying the radius for the collision check during the removal of the elements
    #: for the milling criteria.
    radius: float | None = None

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        millingDirections: tuple,
        region: Region,
        csys: int | None = None,
        millingCheckRegion: Literal[C.MILLING_REGION] = MILLING_REGION,
        radius: float | None = None,
    ):
        """This method creates a TopologyMillingControl object.

        .. note::
            This function can be accessed by::

                mdb.models[name].optimizationTasks[name].TopologyMillingControl

        Parameters
        ----------
        name
            A String specifying the geometric restriction repository key.
        millingDirections
            A tuple of VertexArray objects of length 2 specifying the milling directions. Each point
            can be specified through a tuple of coordinates instead of through a ConstrainedSketchVertex.
        region
            A Region object specifying the region to which the geometric restriction is applied.
        csys
            None or a DatumCsys object specifying the local coordinate system of the
            **millingDirections**. If **csys** = None, the global coordinate system is used. When this
            member is queried, it returns an Int indicating the identifier of the DatumCsys. The
            default value is None.
        millingCheckRegion
            The SymbolicConstant MILLING_REGION or a Region object specifying the milling check
            region. If the value is MILLING_REGION, the value of **region** is used as both the
            milling control region and the milling check region. The default value is
            MILLING_REGION.
        radius
            A Float specifying the radius for the collision check during the removal of the elements
            for the milling criteria.

        Returns
        -------
        TopologyMillingControl
            A TopologyMillingControl object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        csys: int | None = None,
        millingCheckRegion: Literal[C.MILLING_REGION] = MILLING_REGION,
        radius: float | None = None,
    ):
        """This method modifies the TopologyMillingControl object.

        Parameters
        ----------
        csys
            None or a DatumCsys object specifying the local coordinate system of the
            **millingDirections**. If **csys** = None, the global coordinate system is used. When this
            member is queried, it returns an Int indicating the identifier of the DatumCsys. The
            default value is None.
        millingCheckRegion
            The SymbolicConstant MILLING_REGION or a Region object specifying the milling check
            region. If the value is MILLING_REGION, the value of **region** is used as both the
            milling control region and the milling check region. The default value is
            MILLING_REGION.
        radius
            A Float specifying the radius for the collision check during the removal of the elements
            for the milling criteria.
        """
        ...



from __future__ import annotations

from regionToolset import Region
from sketch import Sketch, SketchTransform
from xyPlot import (
    XYDataFromFile,
    XYDataFromFreeBody,
    XYDataFromHistory,
    XYDataFromPath,
    XYDataFromShellThickness,
    xyDataListFromField,
)

# inspected from Abaqus cli

__all__ = [
    "Region",
    "Sketch",
    "SketchTransform",
    "XYDataFromFile",
    "XYDataFromFreeBody",
    "XYDataFromHistory",
    "XYDataFromPath",
    "XYDataFromShellThickness",
    "xyDataListFromField",
    "XYDataFromFile",
    "XYDataFromFreeBody",
    "XYDataFromHistory",
    "XYDataFromPath",
    "XYDataFromShellThickness",
    "xyDataListFromField",
]



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import STEP, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .Amplitude import Amplitude


@abaqus_class_doc
class DecayAmplitude(Amplitude):
    """The DecayAmplitude object defines an amplitude curve using an exponential decay. The DecayAmplitude
    object is derived from the Amplitude object.

    .. note::
        This object can be accessed by::

            import amplitude
            mdb.models[name].amplitudes[name]
            import odbAmplitude
            session.odbs[name].amplitudes[name]

        The corresponding analysis keywords are:

        - AMPLITUDE
    """

    #: A String specifying the repository key.
    name: str

    #: A Float specifying the constant A0A0.
    initial: float

    #: A Float specifying the coefficient AA.
    maximum: float

    #: A Float specifying the starting time t0t0. Possible values are non-negative numbers.
    start: float

    #: A Float specifying the decay time tdtd. Possible values are non-negative numbers.
    decayTime: float

    #: A SymbolicConstant specifying the time span of the amplitude. Possible values are STEP
    #: and TOTAL. The default value is STEP.
    timeSpan: SymbolicConstant = STEP

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        initial: float,
        maximum: float,
        start: float,
        decayTime: float,
        timeSpan: Literal[C.STEP, C.TOTAL] = STEP,
    ):
        """This method creates a DecayAmplitude object.

        .. note::
            This function can be accessed by::

                mdb.models[name].DecayAmplitude
                session.odbs[name].DecayAmplitude

        Parameters
        ----------
        name
            A String specifying the repository key.
        initial
            A Float specifying the constant A0A0.
        maximum
            A Float specifying the coefficient AA.
        start
            A Float specifying the starting time t0t0. Possible values are non-negative numbers.
        decayTime
            A Float specifying the decay time tdtd. Possible values are non-negative numbers.
        timeSpan
            A SymbolicConstant specifying the time span of the amplitude. Possible values are STEP
            and TOTAL. The default value is STEP.

        Returns
        -------
        DecayAmplitude
            A DecayAmplitude object.

        Raises
        ------
        InvalidNameError
        RangeError
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, timeSpan: Literal[C.STEP, C.TOTAL] = STEP):
        """This method modifies the DecayAmplitude object.

        Parameters
        ----------
        timeSpan
            A SymbolicConstant specifying the time span of the amplitude. Possible values are STEP
            and TOTAL. The default value is STEP.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import DOF_MODE, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .Section import Section


@abaqus_class_doc
class MPCSection(Section):
    """The MPCSection object defines the properties of a multi-point constraint section. The MPCSection object
    is derived from the Section object.

    .. note::
        This object can be accessed by::

            import section
            mdb.models[name].sections[name]
            import odbSection
            session.odbs[name].sections[name]

        The corresponding analysis keywords are:

        - MPC
    """

    #: A String specifying the repository key.
    name: str

    #: A SymbolicConstant specifying the MPC type of the section. Possible values are BEAM_MPC,
    #: ELBOW_MPC, PIN_MPC, LINK_MPC, TIE_MPC, and USER_DEFINED.
    mpcType: SymbolicConstant

    #: A SymbolicConstant specifying the mode of the MPC when it is user-defined. Possible
    #: values are DOF_MODE and NODE_MODE. The default value is DOF_MODE.The **userMode** argument
    #: applies only when **mpcType** = USER_DEFINED.
    userMode: SymbolicConstant = DOF_MODE

    #: An Int specifying to differentiate between different constraint types in a user-defined
    #: MPCSection. The default value is 0.The **userType** argument applies only when
    #: **mpcType** = USER_DEFINED.
    userType: int = 0

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        mpcType: Literal[C.BEAM_MPC, C.ELBOW_MPC, C.TIE_MPC, C.MPC, C.USER_DEFINED, C.PIN_MPC, C.LINK_MPC],
        userMode: Literal[C.USER_DEFINED, C.MPC, C.NODE_MODE, C.DOF_MODE] = DOF_MODE,
        userType: int = 0,
    ):
        """This method creates a MPCSection object.

        .. note::
            This function can be accessed by::

                mdb.models[name].MPCSection
                session.odbs[name].MPCSection

        Parameters
        ----------
        name
            A String specifying the repository key.
        mpcType
            A SymbolicConstant specifying the MPC type of the section. Possible values are BEAM_MPC,
            ELBOW_MPC, PIN_MPC, LINK_MPC, TIE_MPC, and USER_DEFINED.
        userMode
            A SymbolicConstant specifying the mode of the MPC when it is user-defined. Possible
            values are DOF_MODE and NODE_MODE. The default value is DOF_MODE.The **userMode** argument
            applies only when **mpcType** = USER_DEFINED.
        userType
            An Int specifying to differentiate between different constraint types in a user-defined
            MPCSection. The default value is 0.The **userType** argument applies only when
            **mpcType** = USER_DEFINED.

        Returns
        -------
        MPCSection
            A MPCSection object.

        Raises
        ------
        RangeError and InvalidNameError
        """
        super().__init__()



from __future__ import annotations

from typing import Any

from .AFXDataComponent import AFXDataComponent
from .constants import DEFAULT_PAD
from .FXComposite import FXComposite
from .FXFont import FXFont
from .FXObject import FXObject
from .FXPacker import FXPacker

#: Use a check button instead of a label.
AFXCOMBOBOX_CHECKBUTTON: int = hash("AFXCOMBOBOX_CHECKBUTTON")

#: Use a radio button instead of a label.
AFXCOMBOBOX_RADIOBUTTON: int = hash("AFXCOMBOBOX_RADIOBUTTON")

#: Orient label or button above combo box.
AFXCOMBOBOX_VERTICAL: int = hash("AFXCOMBOBOX_VERTICAL")

#: Allow interaction with float keywords.
AFXCOMBOBOX_FLOAT: int = hash("AFXCOMBOBOX_FLOAT")

#: Configure combo box to the read-only state.
AFXCOMBOBOX_READONLY: int = hash("AFXCOMBOBOX_READONLY")

#: Include spinner buttons.
AFXCOMBOBOX_SPINNER: int = hash("AFXCOMBOBOX_SPINNER")


class AFXComboBox(FXPacker, AFXDataComponent):
    """This class implements a labeled combo box.

    It allows the user to select entries from a drop-down list. Each item has associated data set as
    integers as items are added.
    """

    #: Label or button ID.
    ID_BUTTON: int = hash("ID_BUTTON")

    #: Combo box ID.
    ID_COMBO: int = hash("ID_COMBO")

    #: Up arrow button ID.
    ID_INCREMENT: int = hash("ID_INCREMENT")

    #: Down arrow button ID.
    ID_DECREMENT: int = hash("ID_DECREMENT")

    def __init__(
        self,
        p: FXComposite,
        ncols: int,
        nvis: int,
        text: str,
        tgt: FXObject | None = None,
        sel: int = 0,
        opts: int = 0,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
        pl: int = DEFAULT_PAD,
        pr: int = DEFAULT_PAD,
        pt: int = DEFAULT_PAD,
        pb: int = DEFAULT_PAD,
    ):
        """Constructor.

        Parameters
        ----------
        p : FXComposite
            Parent widget.
        ncols : int
            Number of columns in the combo box (use 0 for auto-sizing).
        nvis : int
            Number of visible items in the combo box's drop down list.
        text : str
            Label string.
        tgt : FXObject | None
            Message target.
        sel : int
            Message ID.
        opts : int
            Options and hints.
        x : int
            X coordinate of origin.
        y : int
            Y coordinate of origin.
        w : int
            Width of the widget.
        h : int
            Height of the widget.
        pl : int
            Left padding (margin).
        pr : int
            Right padding (margin).
        pt : int
            Top padding (margin).
        pb : int
            Bottom padding (margin).
        """

    def appendItem(self, text: str, sel: int = 0):
        """Adds an item to the end of the list.

        Parameters
        ----------
        text : str
            Text.
        sel : int
            Integer associated with this item.
        """

    def clearItems(self):
        """Removes all items from the list."""

    def create(self):
        """Creates the combo box.

        Reimplemented from FXComposite.
        """

    def disable(self):
        """Disables the combo box.

        Reimplemented from FXWindow.
        """

    def enable(self):
        """Enables the combo box.

        Reimplemented from FXWindow.
        """

    def getCheck(self):
        """Returns the state of the check button or the radio button."""

    def getCurrentItem(self):
        """Returns the index of the current item."""

    def getHelpText(self):
        """Returns the status line help text."""

    def getItemData(self, index: int):
        """Returns the data for the specified item.

        Parameters
        ----------
        index : int
            Index.
        """

    def getItemIndexForData(self, data: Any):
        """Returns the index of the first item with the associated data or -1 if not found.

        Parameters
        ----------
        data : Any
        """

    def getItemIndexForFloat(self, val: float):
        """Returns the index of the first item with the text evaluating to the given value.

        Parameters
        ----------
        val : float
        """

    def getItemProvider(self):
        """Returns the provider of the combo box's items."""

    def getItemText(self, index: int):
        """Returns the text for the specified item.

        Parameters
        ----------
        index : int
            Index.
        """

    def getLabelFont(self):
        """Returns the label font."""

    def getLabelText(self):
        """Returns the label string."""

    def getNumColumns(self):
        """Returns the number of columns."""

    def getNumItems(self):
        """Returns the number of items in the list."""

    def getNumVisible(self):
        """Returns the number of visible items."""

    def getText(self):
        """Returns the text displayed in input field."""

    def getTipText(self):
        """Returns the tool tip message."""

    def insertItem(self, index: int, text: str, sel: int = 0):
        """Inserts a new item at the specified index position.

        Parameters
        ----------
        index : int
            Index.
        text : str
            Text.
        sel : int
            Integer associated with this item
        """

    def isEditable(self):
        """Returns True if the text in the input field may be edited."""

    def isItemCurrent(self, index: int):
        """Returns True if the item at the specified index position is the current item.

        Parameters
        ----------
        index : int
            Index.
        """

    def isReadOnlyState(self):
        """Returns True if the combo box appears in the read-only state."""

    def removeItem(self, index: int):
        """Removes the item at the specified index position from the list.

        Parameters
        ----------
        index : int
            Index.
        """

    def replaceItem(self, index: int, text: str, sel: int = 0):
        """Replaces the item at the specified index position.

        Parameters
        ----------
        index : int
            Index.
        text : str
            Text.
        sel : int
            Integer associated with this item
        """

    def setCheck(self, state: bool):
        """Sets the state of the check button or the radio button.

        Parameters
        ----------
        state : bool
            Button state.
        """

    def setCheckButtonSelector(self, sel: int):
        """Sets the message ID of the check button or the radio button.

        Parameters
        ----------
        sel : int
            Selector.
        """

    def setCheckButtonTarget(self, tgt: FXObject, checkVal: bool = False):
        """Sets the message target of the check button or the radio button.

        Parameters
        ----------
        tgt : FXObject
            Target.
        checkVal : bool
            Value of check button.
        """

    def setCurrentItem(self, index: int):
        """Sets the current item (the index is zero-based).

        Parameters
        ----------
        index : int
            Index.
        """

    def setEditable(self, edit: bool = True):
        """Sets the editable state for the input field.

        Parameters
        ----------
        edit : bool
            Editable state.
        """

    def setFocusAndSelection(self):
        """Moves the focus to the input field and selects its contents if the combo box is editable."""

    def setFocusToCheckButton(self):
        """Moves the focus to the check button or the radio button (if existed) of the widget."""

    def setFocusToComboBox(self):
        """Moves the focus to the input field of the widget."""

    def setHelpText(self, text: str):
        """Sets the status line help text.

        Parameters
        ----------
        text : str
            Help text.
        """

    def setItemData(self, index: int, ptr: str):
        """Sets the data for the specified item.

        Parameters
        ----------
        index : int
            Index.
        ptr : str
            Data.
        """

    def setItemProvider(self, cp: FXObject):
        """Sets the provider of this object items.

        Parameters
        ----------
        cp : FXObject
            Item provider.
        """

    def setItemText(self, index: int, text: str):
        """Sets the text for the specified item.

        Parameters
        ----------
        index : int
            Index.
        text : str
            Text.
        """

    def setLabelFont(self, fnt: FXFont):
        """Sets the label font.

        Parameters
        ----------
        fnt : FXFont
            Label font.
        """

    def setLabelText(self, txt: str):
        """Sets the label string.

        Parameters
        ----------
        txt : str
            Label text.
        """

    def setMaxVisible(self, maxVis: int):
        """Sets the maximum number of visible items. The combo box will show up to the given maximum number of
        items in its list. If the combo box has more items, its list will show a scroll bar.

        Parameters
        ----------
        maxVis : int
            Maximum number of visible items.
        """

    def setNumColumns(self, cols: int):
        """Sets the number of columns in the combo box; passing zero will cause the combo box to always have the
        number of columns equal to the maximum item length.

        Parameters
        ----------
        cols : int
            Number of columns.
        """

    def setNumVisible(self, nvis: int):
        """Sets the number of visible items.

        Parameters
        ----------
        nvis : int
            Number of visible items.
        """

    def setReadOnlyState(self, readonly: bool = True):
        """Sets the read-only state of the combo box.

        Parameters
        ----------
        readonly : bool
            Read-only state.
        """

    def setText(self, txt: str):
        """Sets the text displayed in the input field.

        Parameters
        ----------
        txt : str
            Input field text.
        """

    def setTipText(self, text: str):
        """Sets the tool tip message.

        Parameters
        ----------
        text : str
            Tooltip text.
        """



from __future__ import annotations

from .FXChore import FXChore
from .FXColor import FXColor
from .FXFont import FXFont
from .FXInputHandle import FXInputHandle
from .FXObject import FXObject
from .FXTimer import FXTimer
from .FXWindow import FXWindow
from .SOCKET import SOCKET

#: Inactive.
INPUT_NONE: int = hash("INPUT_NONE")

#: Read input fd.
INPUT_READ: int = hash("INPUT_READ")

#: Write input fd.
INPUT_WRITE: int = hash("INPUT_WRITE")

#: Except input fd.
INPUT_EXCEPT: int = hash("INPUT_EXCEPT")

#: Non modal event loop (dispatch normally).
MODAL_FOR_NONE: int = hash("MODAL_FOR_NONE")

#: Modal dialog (beep if outside of modal dialog).
MODAL_FOR_WINDOW: int = hash("MODAL_FOR_WINDOW")

#: Modal for popup (always dispatch to popup).
MODAL_FOR_POPUP: int = hash("MODAL_FOR_POPUP")

#: Arrow cursor.
DEF_ARROW_CURSOR: int = hash("DEF_ARROW_CURSOR")

#: Reverse arrow cursor.
DEF_RARROW_CURSOR: int = hash("DEF_RARROW_CURSOR")

#: Text cursor.
DEF_TEXT_CURSOR: int = hash("DEF_TEXT_CURSOR")

#: Horizontal split cursor.
DEF_HSPLIT_CURSOR: int = hash("DEF_HSPLIT_CURSOR")

#: Vertical split cursor.
DEF_VSPLIT_CURSOR: int = hash("DEF_VSPLIT_CURSOR")

#: Cross split cursor.
DEF_XSPLIT_CURSOR: int = hash("DEF_XSPLIT_CURSOR")

#: Color swatch drag cursor.
DEF_SWATCH_CURSOR: int = hash("DEF_SWATCH_CURSOR")

#: Move cursor.
DEF_MOVE_CURSOR: int = hash("DEF_MOVE_CURSOR")

#: Resize horizontal edge.
DEF_DRAGH_CURSOR: int = hash("DEF_DRAGH_CURSOR")

#: Resize vertical edge.
DEF_DRAGV_CURSOR: int = hash("DEF_DRAGV_CURSOR")

#: Resize upper-leftcorner.
DEF_DRAGTL_CURSOR: int = hash("DEF_DRAGTL_CURSOR")

#: Resize bottom-right corner.
DEF_DRAGBR_CURSOR: int = hash("DEF_DRAGBR_CURSOR")

#: Resize upper-right corner.
DEF_DRAGTR_CURSOR: int = hash("DEF_DRAGTR_CURSOR")

#: Resize bottom-left corner.
DEF_DRAGBL_CURSOR: int = hash("DEF_DRAGBL_CURSOR")

#: Drag and drop stop.
DEF_DNDSTOP_CURSOR: int = hash("DEF_DNDSTOP_CURSOR")

#: Drag and drop copy.
DEF_DNDCOPY_CURSOR: int = hash("DEF_DNDCOPY_CURSOR")

#: Drag and drop move.
DEF_DNDMOVE_CURSOR: int = hash("DEF_DNDMOVE_CURSOR")

#: Drag and drop link.
DEF_DNDLINK_CURSOR: int = hash("DEF_DNDLINK_CURSOR")

#: Cross hair cursor.
DEF_CROSSHAIR_CURSOR: int = hash("DEF_CROSSHAIR_CURSOR")

#: North-east cursor.
DEF_CORNERNE_CURSOR: int = hash("DEF_CORNERNE_CURSOR")

#: North-west cursor.
DEF_CORNERNW_CURSOR: int = hash("DEF_CORNERNW_CURSOR")

#: South-east cursor.
DEF_CORNERSE_CURSOR: int = hash("DEF_CORNERSE_CURSOR")

#: South-west cursor.
DEF_CORNERSW_CURSOR: int = hash("DEF_CORNERSW_CURSOR")

#: Rotate cursor.
DEF_ROTATE_CURSOR: int = hash("DEF_ROTATE_CURSOR")


class FXApp(FXObject):
    """Application Object."""

    #: Terminate the application normally.
    ID_QUIT: int = hash("ID_QUIT")

    #: Dump the current widget tree.
    ID_DUMP: int = hash("ID_DUMP")

    def __init__(self, name: str = "Application", vendor: str = "FoxDefault"):
        """Copyright notice of library. Construct application object; the name and vendor strings are used as
        keys into the registry database for this application's settings.

        Parameters
        ----------
        name : str

        vendor : str
        """

    def addChore(self, tgt: FXObject, sel: int):
        """Add a idle processing message to be sent to target object when the system becomes idle, i.e. there
        are no events to be processed.

        Parameters
        ----------
        tgt : FXObject

        sel : int
        """

    def addInput(self, fd: FXInputHandle, mode: int, tgt: FXObject, sel: int):
        """Add a file descriptor fd to be watched for activity as determined by mode, where mode is a bitwise OR
        (INPUT_READ, INPUT_WRITE, INPUT_EXCEPT). A message of type SEL_IO_READ, SEL_IO_WRITE, or SEL_IO_EXCEPT
        will be sent to the target when the specified activity is detected on the file descriptor. On Windows, a
        Win32 event, not a file descriptor, must be specified. The client code for this interface must be
        platform-dependent. See addSocket below for a portable interface. CAE.

        Parameters
        ----------
        fd : FXInputHandle

        mode : int

        tgt : FXObject

        sel : int
        """

    def addSocket(self, sd: SOCKET, mode: int, tgt: FXObject, sel: int):
        """CAE Add a socket descriptor sd to be watched for activity as determined by mode, where mode is a
        bitwise OR (INPUT_READ, INPUT_WRITE, INPUT_EXCEPT). A message of type SEL_IO_READ, SEL_IO_WRITE, or
        SEL_IO_EXCEPT will be sent to the target when the specified activity is detected on the socket
        descriptor. This is identical to addInput on Unix. It behaves the same on Windows.

        Parameters
        ----------
        sd : SOCKET

        mode : int

        tgt : FXObject

        sel : int
        """

    def addTimeout(self, ms: int, tgt: FXObject, sel: int):
        """Add timeout message to be sent to target object in ms milliseconds; the timer fires only once after
        the interval expires.

        Parameters
        ----------
        ms : int

        tgt : FXObject

        sel : int
        """

    def beep(self):
        """Beep."""

    def beginWaitCursor(self):
        """Begin of wait-cursor block; wait-cursor blocks may be nested."""

    def create(self):
        """Create application's windows.

        Reimplemented in AFXApp.
        """

    def endWaitCursor(self):
        """End of wait-cursor block."""

    def forceRefresh(self):
        """Force GUI refresh."""

    def getAppName(self):
        """Get application name."""

    def getBorderColor(self):
        """Obtain default colors."""

    def getMainWindow(self):
        """Get main window, if any."""

    def getMonoVisual(self):
        """Get monochrome visual."""

    def getNormalFont(self):
        """Return default font."""

    def getRoot(self):
        """Get root Window."""

    def getTypingSpeed(self):
        """Obtain application-wide settings."""

    def init(self, argc: int, argv: str, connect: bool = True):
        """Initialize application. Parses and removes common command line arguments, reads the registry.
        Finally, if connect is True, it opens the display.

        Parameters
        ----------
        argc : int

        argv : str

        connect : bool
        """

    def peekEvent(self):
        """Peek to determine if there's an event."""

    def refresh(self):
        """Schedule a refresh."""

    def removeChore(self, c: FXChore):
        """Remove idle processing message.

        Parameters
        ----------
        c : FXChore
        """

    def removeInput(self, fd: FXInputHandle, mode: int):
        """Remove input message and target object for the specified file descriptor and mode, which is a bitwise
        OR of (INPUT_READ, INPUT_WRITE, INPUT_EXCEPT).

        Parameters
        ----------
        fd : FXInputHandle

        mode : int
        """

    def removeSocket(self, sd: SOCKET, mode: int):
        """CAE Remove input message and target object for the specified socket descriptor and mode, which is a
        bitwise OR of (INPUT_READ, INPUT_WRITE, INPUT_EXCEPT).

        Parameters
        ----------
        sd : SOCKET

        mode : int
        """

    def removeTimeout(self, t: FXTimer):
        """Remove timeout, returns NULL.

        Parameters
        ----------
        t : FXTimer
        """

    def repaint(self):
        """Paint all windows marked for repainting.

        On return all the applications windows have been painted.
        """

    def run(self):
        """Run the main application event loop until stop() is called, and return the exit code passed as
        argument to stop().

        Reimplemented in AFXApp.
        """

    def runOneEvent(self):
        """Perform one event dispatch."""

    def runUntil(self, condition: int):
        """Run an event loop till some flag becomes non-zero. Reimplemented in AFXApp.

        Parameters
        ----------
        condition : int
        """

    def runWhileEvents(self, window: FXWindow | None = None):
        """Run event loop while there are events are available in the queue. Returns 1 when all events in the
        queue have been handled, and 0 when the event loop was terminated due to stop() or stopModal(). Except
        for the modal window and its children, user input to all windows is blocked; if the modal window is NULL
        all user input is blocked.

        Parameters
        ----------
        window : FXWindow | None
        """

    def setBorderColor(self, color: FXColor):
        """Change default colors.

        Parameters
        ----------
        color : FXColor
        """

    def setNormalFont(self, font: FXFont):
        """Change default font.

        Parameters
        ----------
        font : FXFont
        """

    def setTypingSpeed(self, speed: int):
        """Change application-wide settings.

        Parameters
        ----------
        speed : int
        """

    def stop(self, value: int = 0):
        """Terminate the outermost event loop, and all inner modal loops; All more deeper nested event loops
        will be terminated with code equal to 0, while the outermost event loop will return code equal to value.

        Parameters
        ----------
        value : int
        """



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import CENTROID, NORMAL_TANGENTIAL
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class FreeBody:
    """The FreeBody object defines a section across which resultant forces and moments are computed.

    .. note::
        This object can be accessed by::

            import visualization
            session.freeBodies[name]
    """

    @abaqus_method_doc
    def FreeBodyFromEdges(
        self,
        name: str,
        edges: str,
        summationLoc: Literal[C.SPECIFY, C.CENTROID, C.NODAL_AVERAGE] = CENTROID,
        summationPoint: tuple = (),
        componentResolution: Literal[C.NORMAL_TANGENTIAL, C.CSYS] = NORMAL_TANGENTIAL,
        csysName: str = "",
    ):
        """This method creates a FreeBody object and places it in the freeBodies repository.

        .. note::
            This function can be accessed by::

                session.FreeBodyFromEdges

        Parameters
        ----------
        name
            A string name for the free body.
        edges
            A DisplayGroup leaf object that specifies the physical constituents of the free body.
        summationLoc
            A SymbolicConstant specifying the location of the summation point. Possible values are
            CENTROID, NODAL_AVERAGE and SPECIFY. The default value is CENTROID.
        summationPoint
            A tuple of 3 floats specifying the summation point.
        componentResolution
            A SymbolicConstant specifying the component resolution. Possible values are
            NORMAL_TANGENTIAL and CSYS. The default value is NORMAL_TANGENTIAL.
        csysName
            A string specifying the name of the coordinate system.

        Returns
        -------
        FreeBody
            A FreeBody object.
        """
        ...

    @abaqus_method_doc
    def FreeBodyFromFaces(
        self,
        name: str,
        faces: str,
        summationLoc: Literal[C.SPECIFY, C.CENTROID, C.NODAL_AVERAGE] = CENTROID,
        summationPoint: tuple = (),
        componentResolution: Literal[C.NORMAL_TANGENTIAL, C.CSYS] = NORMAL_TANGENTIAL,
        csysName: str = "",
    ):
        """This method creates a FreeBody object and places it in the freeBodies repository.

        .. note::
            This function can be accessed by::

                session.FreeBodyFromEdges

        Parameters
        ----------
        name
            A string name for the free body.
        faces
            A DisplayGroup leaf object that specifies the physical constituents of the free body.
        summationLoc
            A SymbolicConstant specifying the location of the summation point. Possible values are
            CENTROID, NODAL_AVERAGE and SPECIFY. The default value is CENTROID.
        summationPoint
            A tuple of 3 floats specifying the summation point.
        componentResolution
            A SymbolicConstant specifying the component resolution. Possible values are
            NORMAL_TANGENTIAL and CSYS. The default value is NORMAL_TANGENTIAL.
        csysName
            A string specifying the name of the coordinate system.

        Returns
        -------
        FreeBody
            A FreeBody object.
        """
        ...

    @abaqus_method_doc
    def FreeBodyFromNodesElements(
        self,
        name: str,
        elements: str,
        nodes: str,
        summationLoc: Literal[C.SPECIFY, C.CENTROID, C.NODAL_AVERAGE] = CENTROID,
        summationPoint: tuple = (),
        componentResolution: Literal[C.NORMAL_TANGENTIAL, C.CSYS] = NORMAL_TANGENTIAL,
        csysName: str = "",
    ):
        """This method creates a FreeBody object and places it in the freeBodies repository.

        .. note::
            This function can be accessed by::

                session.FreeBodyFromEdges

        Parameters
        ----------
        name
            A string name for the free body.
        elements
            A DisplayGroup leaf object that specifies the physical constituents of the free body.
        nodes
            A DisplayGroup leaf object that specifies the physical constituents of the free body.
        summationLoc
            A SymbolicConstant specifying the location of the summation point. Possible values are
            CENTROID, NODAL_AVERAGE and SPECIFY. The default value is CENTROID.
        summationPoint
            A tuple of 3 floats specifying the summation point.
        componentResolution
            A SymbolicConstant specifying the component resolution. Possible values are
            NORMAL_TANGENTIAL and CSYS. The default value is NORMAL_TANGENTIAL.
        csysName
            A string specifying the name of the coordinate system.

        Returns
        -------
        FreeBody
            A FreeBody object.
        """
        ...



from __future__ import annotations

from typing import Union

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import ELEMENT, OFF, ON, Boolean, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class ProbeOptions:
    """The ProbeOptions object is used to store settings associated with probing a model or an.

    **X - Y** plot. The ProbeOptions object has no constructor. Abaqus creates the
    **defaultProbeOptions** and the **probeOptions** members when you import the Visualization
    module. When probing is initiated for the first time, the values in the **probeOptions**
    member are initialized using the values from the **defaultProbeOptions** member.

    .. note::
        This object can be accessed by::

            import visualization
            session.defaultProbeOptions
            session.probeOptions
    """

    #: A SymbolicConstant specifying the entity being probed when **probeObject** = "ODB". Possible
    #: values are NODE and ELEMENT. The default value is ELEMENT.
    probeEntity: SymbolicConstant = ELEMENT

    #: A SymbolicConstant specifying the output position to probe for field output results when
    #: **probeObject** = "ODB". Possible values are NODAL, INTEGRATION_POINT, ELEMENT_FACE,
    #: ELEMENT_NODAL, and ELEMENT_CENTROID.When **probeEntity** = NODE, the only possible value is
    #: NODAL. When **probeEntity** = ELEMENT, the only possible values are INTEGRATION_POINT,
    #: ELEMENT_FACE, ELEMENT_NODAL, and ELEMENT_CENTROID, and the default is INTEGRATION_POINT.
    probeOutputPosition: SymbolicConstant

    #: A Boolean specifying whether to display the part instance information. This member is
    #: valid when **probeObject** = ODB. The default value is ON.
    partInstance: Boolean = ON

    #: A Boolean specifying whether to display the element ID information. This member is valid
    #: when **probeObject** = ODB and **probeEntity** = ELEMENT. The default value is ON.
    elementID: Boolean = ON

    #: A Boolean specifying whether to display the element type information. This member is
    #: valid when **probeObject** = ODB and **probeEntity** = ELEMENT. The default value is ON.
    elementType: Boolean = ON

    #: A Boolean specifying whether to display the element connectivity. This member is valid
    #: when **probeObject** = ODB and **probeEntity** = ELEMENT. The default value is ON.
    elementConnectivity: Boolean = ON

    #: A Boolean specifying whether to display the element field output results. This member is
    #: valid when **probeObject** = ODB, **probeEntity** = ELEMENT, and **isFieldOutputAvailable** = ON.
    #: The default value is ON.
    elementFieldResults: Boolean = ON

    #: A Boolean specifying whether to display the node ID when **probeObject** = ODB and
    #: **probeEntity** = NODE. The default value is ON.
    nodeId: Boolean = ON

    #: A Boolean specifying whether to display the base coordinates of a node when
    #: **probeObject** = ODB and **probeEntity** = NODE. The default value is ON.
    baseCoordinates: Boolean = ON

    #: A Boolean specifying whether to display the deformed coordinates of a node when
    #: **probeObject** = ODB and **probeEntity** = NODE. The default value is ON.
    deformedCoordinates: Boolean = ON

    #: A Boolean specifying whether to display the elements attached to a node when
    #: **probeObject** = ODB and **probeEntity** = NODE. The default value is ON.
    attachedElements: Boolean = ON

    #: A Boolean specifying whether to display the node field output results. This member is
    #: valid when **probeObject** = ODB, **probeEntity** = NODE, and **isFieldOutputAvailable** = ON. The
    #: default value is ON.
    nodeFieldResults: Boolean = ON

    #: A Boolean specifying whether to display the legend for a curve being probed. This member
    #: is valid when **probeObject** = XYPlot. The default value is ON.
    legend: Boolean = ON

    #: A Boolean specifying whether to display the **X** coordinate value of the point on the
    #: curve being probed. This member is valid when **probeObject** = XYPlot. The default value is
    #: ON.
    xValue: Boolean = ON

    #: A Boolean specifying whether to display the **Y** coordinate value of the point on the
    #: curve being probed. This member is valid when **probeObject** = XYPlot. The default value is
    #: ON.
    yValue: Boolean = ON

    #: A Boolean specifying whether to display the sequence ID of the point on the curve being
    #: probed. This member is valid when **probeObject** = XYPlot. The default value is ON.
    sequenceID: Boolean = ON

    #: A Boolean specifying whether to interpolate values within a line segment when
    #: **probeObject** = XYPlot. When **interpolateXy** = OFF, probing returns the nearest **X - Y** data
    #: point on the curve. When **interpolateXy** = ON, probing interpolates data to return a value
    #: at the nearest point on the curve. The default value is OFF.
    interpolateXy: Boolean = OFF

    #: A Boolean specifying whether field output is available for probing when
    #: **probeObject** = XYPlot. This member is read-only.
    isFieldOutputAvailable: Boolean = OFF

    #: A String specifying the type of the displayed object being probed. Possible values are
    #: "ODB" and "XYPlot". This member is read-only.
    probeObject: str = ""

    @abaqus_method_doc
    def setValues(
        self,
        options: Union["ProbeOptions", None] = None,
        probeEntity: Literal[C.NODE, C.ODB, C.ELEMENT] = ELEMENT,
        probeOutputPosition: (
            Literal[
                C.ELEMENT_NODAL,
                C.ELEMENT_FACE,
                C.ELEMENT,
                C.NODAL,
                C.INTEGRATION_POINT,
                C.NODE,
                C.ODB,
                C.ELEMENT_CENTROID,
            ]
            | None
        ) = None,
        partInstance: Boolean = ON,
        elementID: Boolean = ON,
        elementType: Boolean = ON,
        elementConnectivity: Boolean = ON,
        elementFieldResults: Boolean = ON,
        nodeId: Boolean = ON,
        baseCoordinates: Boolean = ON,
        deformedCoordinates: Boolean = ON,
        attachedElements: Boolean = ON,
        nodeFieldResults: Boolean = ON,
        legend: Boolean = ON,
        xValue: Boolean = ON,
        yValue: Boolean = ON,
        sequenceID: Boolean = ON,
        interpolateXy: Boolean = OFF,
    ):
        """This method modifies the settings on the ProbeOptions object.

        Parameters
        ----------
        options
            A ProbeOptions object from which values are to be copied. If other arguments are also
            supplied to setValues, they will override the values in **options**. The default value is
            None.
        probeEntity
            A SymbolicConstant specifying the entity being probed when **probeObject** = "ODB". Possible
            values are NODE and ELEMENT. The default value is ELEMENT.
        probeOutputPosition
            A SymbolicConstant specifying the output position to probe for field output results when
            **probeObject** = "ODB". Possible values are NODAL, INTEGRATION_POINT, ELEMENT_FACE,
            ELEMENT_NODAL, and ELEMENT_CENTROID.When **probeEntity** = NODE, the only possible value is
            NODAL. When **probeEntity** = ELEMENT, the only possible values are INTEGRATION_POINT,
            ELEMENT_FACE, ELEMENT_NODAL, and ELEMENT_CENTROID, and the default is INTEGRATION_POINT.
        partInstance
            A Boolean specifying whether to display the part instance information. This member is
            valid when **probeObject** = ODB. The default value is ON.
        elementID
            A Boolean specifying whether to display the element ID information. This member is valid
            when **probeObject** = ODB and **probeEntity** = ELEMENT. The default value is ON.
        elementType
            A Boolean specifying whether to display the element type information. This member is
            valid when **probeObject** = ODB and **probeEntity** = ELEMENT. The default value is ON.
        elementConnectivity
            A Boolean specifying whether to display the element connectivity. This member is valid
            when **probeObject** = ODB and **probeEntity** = ELEMENT. The default value is ON.
        elementFieldResults
            A Boolean specifying whether to display the element field output results. This member is
            valid when **probeObject** = ODB, **probeEntity** = ELEMENT, and **isFieldOutputAvailable** = ON.
            The default value is ON.
        nodeId
            A Boolean specifying whether to display the node ID when **probeObject** = ODB and
            **probeEntity** = NODE. The default value is ON.
        baseCoordinates
            A Boolean specifying whether to display the base coordinates of a node when
            **probeObject** = ODB and **probeEntity** = NODE. The default value is ON.
        deformedCoordinates
            A Boolean specifying whether to display the deformed coordinates of a node when
            **probeObject** = ODB and **probeEntity** = NODE. The default value is ON.
        attachedElements
            A Boolean specifying whether to display the elements attached to a node when
            **probeObject** = ODB and **probeEntity** = NODE. The default value is ON.
        nodeFieldResults
            A Boolean specifying whether to display the node field output results. This member is
            valid when **probeObject** = ODB, **probeEntity** = NODE, and **isFieldOutputAvailable** = ON. The
            default value is ON.
        legend
            A Boolean specifying whether to display the legend for a curve being probed. This member
            is valid when **probeObject** = XYPlot. The default value is ON.
        xValue
            A Boolean specifying whether to display the **X** coordinate value of the point on the
            curve being probed. This member is valid when **probeObject** = XYPlot. The default value is
            ON.
        yValue
            A Boolean specifying whether to display the **Y** coordinate value of the point on the
            curve being probed. This member is valid when **probeObject** = XYPlot. The default value is
            ON.
        sequenceID
            A Boolean specifying whether to display the sequence ID of the point on the curve being
            probed. This member is valid when **probeObject** = XYPlot. The default value is ON.
        interpolateXy
            A Boolean specifying whether to interpolate values within a line segment when
            **probeObject** = XYPlot. When **interpolateXy** = OFF, probing returns the nearest **X - Y** data
            point on the curve. When **interpolateXy** = ON, probing interpolates data to return a value
            at the nearest point on the curve. The default value is OFF.
        """
        ...



from __future__ import annotations

from typing import List, Type

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .Annotation import Annotation


@abaqus_class_doc
class AnnotationsToPlotArray(List[Annotation]):
    """The AnnotationsToPlotArray object is a sequence that stores references to plotted annotations. By adding
    annotations to and removing annotations from this sequence, you can control which annotations are displayed
    in a particular viewport.

    .. note::
        This object can be accessed by::

            import annotationToolset
            session.viewports[name].annotationsToPlot
    """

    def __new__(cls: Type[AnnotationsToPlotArray]) -> AnnotationsToPlotArray:
        return super().__new__(cls)

    @abaqus_method_doc
    def bringForward(self, index: str):
        """This method brings the Annotation object one position forward in the AnnotationsToPlotArray sequence.

        Parameters
        ----------
        index
            An Int specifying the index of the Annotation object in the AnnotationsToPlotArray
            sequence.
        """
        # TODO: implement this method
        ...

    @abaqus_method_doc
    def bringToFront(self, index: str):
        """This method brings the Annotation object to the beginning of the AnnotationsToPlotArray sequence.

        Parameters
        ----------
        index
            An Int specifying the index of the Annotation object in the AnnotationsToPlotArray
            sequence.
        """
        # TODO: implement this method
        ...

    @abaqus_method_doc
    def moveAfter(self, index: str, other: str):
        """This method moves the Annotation object after another object in the same AnnotationsToPlotArray
        sequence.

        Parameters
        ----------
        index
            An Integer specifying the index of the Annotation object in the AnnotationsToPlotArray
            sequence.
        other
            An Integer specifying the index of the other Annotation object in the
            AnnotationsToPlotArray sequence after which this object will be moved.
        """
        # TODO: implement this method
        ...

    @abaqus_method_doc
    def moveBefore(self, index: str, other: str):
        """This method moves the Annotation object before another object in the same AnnotationsToPlotArray
        sequence.

        Parameters
        ----------
        index
            An Int specifying the index of the Annotation object in the AnnotationsToPlotArray
            sequence.
        other
            An Int specifying the index of the other Annotation object in the AnnotationsToPlotArray
            sequence before which this object will be moved.
        """
        # TODO: implement this method
        ...

    @abaqus_method_doc
    def sendBackward(self, index: str):
        """This method sends the Annotation object one position backward in the AnnotationsToPlotArray sequence.

        Parameters
        ----------
        index
            An Int specifying the index of the Annotation object in the AnnotationsToPlotArray
            sequence.
        """
        # TODO: implement this method
        ...

    @abaqus_method_doc
    def sendToBack(self, index: str):
        """This method sends the Annotation object to the end of the AnnotationsToPlotArray sequence.

        Parameters
        ----------
        index
            An Int specifying the index of the Annotation object in the AnnotationsToPlotArray
            sequence.
        """
        # TODO: implement this method
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc


@abaqus_class_doc
class UserDefinedField:
    """The UserDefinedField object redefines field variables at a material point.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].userDefinedField
            import odbMaterial
            session.odbs[name].materials[name].userDefinedField

        The corresponding analysis keywords are:

        - USER DEFINED FIELD
    """

    @abaqus_method_doc
    def __init__(self):
        """This method defines a UserDefinedField object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].UserDefinedField
                session.odbs[name].materials[name].UserDefinedField

        Returns
        -------
        UserDefinedField
            A UserDefinedField object.
        """
        ...



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..ConstrainedSketchGeometry.ConstrainedSketchGeometry import (
    ConstrainedSketchGeometry,
)
from .ConstrainedSketchConstraint import ConstrainedSketchConstraint


@abaqus_class_doc
class VerticalConstraint(ConstrainedSketchConstraint):
    @abaqus_method_doc
    def __init__(self, entity: ConstrainedSketchGeometry):
        """This method creates a vertical constraint. This constraint applies to a line and constrains it to be
        vertical.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].VerticalConstraint

        Parameters
        ----------
        entity
            A ConstrainedSketchGeometry object specifying the line to constrain.

        Returns
        -------
        ConstrainedSketchConstraint
            A ConstrainedSketchConstraint object.
        """
        ...



from __future__ import annotations

from .FXLabel import FXLabel

#: Automatically gray out when no target.
MENUBUTTON_AUTOGRAY: int = hash("MENUBUTTON_AUTOGRAY")

#: Automatically hide when no target.
MENUBUTTON_AUTOHIDE: int = hash("MENUBUTTON_AUTOHIDE")

#: Toolbar style.
MENUBUTTON_TOOLBAR: int = hash("MENUBUTTON_TOOLBAR")

#: CAE - Combobox style.
MENUBUTTON_COMBOBOX: int = hash("MENUBUTTON_COMBOBOX")

#: Popup window appears below menu button.
MENUBUTTON_DOWN: int = hash("MENUBUTTON_DOWN")

#: Popup window appears above menu button.
MENUBUTTON_UP: int = hash("MENUBUTTON_UP")

#: Popup window to the left of the menu button.
MENUBUTTON_LEFT: int = hash("MENUBUTTON_LEFT")

#: Popup window to the right of the menu button.
MENUBUTTON_RIGHT: int = hash("MENUBUTTON_RIGHT")

#: Do not show arrows.
MENUBUTTON_NOARROWS: int = hash("MENUBUTTON_NOARROWS")

#: Popup attaches to the left side of the menu button.
MENUBUTTON_ATTACH_LEFT: int = hash("MENUBUTTON_ATTACH_LEFT")

#: Popup attaches to the top of the menu button.
MENUBUTTON_ATTACH_TOP: int = hash("MENUBUTTON_ATTACH_TOP")

#: Popup attaches to the right side of the menu button.
MENUBUTTON_ATTACH_RIGHT: int = hash("MENUBUTTON_ATTACH_RIGHT")

#: Popup attaches to the bottom of the menu button.
MENUBUTTON_ATTACH_BOTTOM: int = hash("MENUBUTTON_ATTACH_BOTTOM")

#: Popup attaches to the center of the menu button.
MENUBUTTON_ATTACH_CENTER: int = hash("MENUBUTTON_ATTACH_CENTER")

#: Popup attaches to both sides of the menu button.
MENUBUTTON_ATTACH_BOTH: int = hash("MENUBUTTON_ATTACH_BOTH")


class FXMenuButton(FXLabel):
    """A menu button posts a popup menu when clicked.

    There are many ways to control the placement where the popup will appear; first, the popup may be placed
    on either of the four sides relative to the menu button; this is controlled by the flags
    MENUBUTTON_DOWN, etc. Next, there are several attachment modes; the popup's left/bottom edge may attach
    to the menu button's left/top edge, or the popup's right/top edge may attach to the menu button's
    right/bottom edge, or both. Also, the popup may apear centered relative to the menu button. Finally, a
    small offset may be specified to displace the location of the popup by a few pixels so as to account for
    borders and so on. Normally, the menu button shows an arrow pointing to the direction where the popup is
    set to appear; this can be turned off by passing the option MENUBUTTON_NOARROWS.
    """



from __future__ import annotations

from .constants import DEFAULT_SPACING
from .FXComposite import FXComposite
from .FXObject import FXObject
from .FXTabBar import FXTabBar

#: Normal tab book style.
TABBOOK_NORMAL: int = hash("TABBOOK_NORMAL")


class FXTabBook(FXTabBar):
    """The tab book layout manager arranges pairs of children; the even numbered children (0,2,4,...) are
    usually tab items, and are placed on the top.

    The odd numbered children are usually layout managers, and are placed below; all the odd numbered
    children are placed on top of each other, similar to the switcher widget. When the user presses one of
    the tab items, the tab item is raised above the neighboring tabs, and the corresponding panel is raised
    to the top. Thus, a tab book can be used to present many GUI controls in a small space by placing
    several panels on top of each other and using tab items to select the desired panel.
    """

    def __init__(
        self,
        p: FXComposite,
        tgt: FXObject | None = None,
        sel: int = 0,
        opts: int = TABBOOK_NORMAL,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
        pl: int = DEFAULT_SPACING,
        pr: int = DEFAULT_SPACING,
        pt: int = DEFAULT_SPACING,
        pb: int = DEFAULT_SPACING,
    ):
        """Construct tab book.

        Parameters
        ----------
        p : FXComposite

        tgt : FXObject | None

        sel : int

        opts : int

        x : int

        y : int

        w : int

        h : int

        pl : int

        pr : int

        pt : int

        pb : int
        """

    def getDefaultHeight(self):
        """Return default height.

        Reimplemented from FXTabBar.
        """

    def getDefaultWidth(self):
        """Return default width.

        Reimplemented from FXTabBar.
        """
